# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from unittest import TestCase

import numpy as np
import pandas as pd
import pandas.testing as pdt

from tests.fixtures import DataTestCase
from tsfresh import extract_features, extract_relevant_features, select_features
from tsfresh.feature_extraction.settings import ComprehensiveFCParameters
from tsfresh.utilities.dataframe_functions import impute


class RelevantFeatureExtractionDataTestCase(DataTestCase):
    """
    Test case for the relevant_feature_extraction function
    """

    def test_functional_equality(self):
        """
        `extract_relevant_features` should be equivalent to running first `extract_features` with impute and
        `select_features` afterwards.
        Meaning it should produce the same relevant features and the values of these features should be identical.
        :return:
        """
        df, y = self.create_test_data_sample_with_target()

        relevant_features = extract_relevant_features(
            df,
            y,
            column_id="id",
            column_value="val",
            column_kind="kind",
            column_sort="sort",
        )

        extracted_features = extract_features(
            df,
            column_id="id",
            column_value="val",
            column_kind="kind",
            column_sort="sort",
            impute_function=impute,
        )
        selected_features = select_features(extracted_features, y)

        self.assertEqual(
            set(relevant_features.columns),
            set(selected_features.columns),
            "Should select the same columns:\n\t{}\n\nvs.\n\n\t{}".format(
                relevant_features.columns, selected_features.columns
            ),
        )

        relevant_columns = relevant_features.columns
        relevant_index = relevant_features.index
        self.assertTrue(
            relevant_features.equals(
                selected_features.loc[relevant_index][relevant_columns]
            ),
            "Should calculate the same feature values",
        )


class RelevantFeatureExtractionTestCase(TestCase):
    def setUp(self):
        np.random.seed(42)
        y = pd.Series(np.random.binomial(1, 0.5, 20), index=range(20))
        df = pd.DataFrame(index=range(100))

        df["a"] = np.random.normal(0, 1, 100)
        df["b"] = np.random.normal(0, 1, 100)
        df["id"] = np.repeat(range(20), 5)

        X = pd.DataFrame(index=range(20))
        X["f1"] = np.random.normal(0, 1, 20)
        X["f2"] = np.random.normal(0, 1, 20)

        self.df = df
        self.X = X
        self.y = y

    def test_extracted_features_contain_X_features(self):
        X = extract_relevant_features(self.df, self.y, self.X, column_id="id")
        self.assertIn("f1", X.columns)
        self.assertIn("f2", X.columns)
        pdt.assert_series_equal(self.X["f1"], X["f1"])
        pdt.assert_series_equal(self.X["f2"], X["f2"])
        pdt.assert_index_equal(self.X["f1"].index, X["f1"].index)
        pdt.assert_index_equal(self.X["f2"].index, X["f2"].index)

    def test_extraction_null_as_column_name(self):

        df1 = pd.DataFrame(
            data={
                0: range(10),
                1: np.repeat([0, 1], 5),
                2: np.repeat([0, 1, 2, 3, 4], 2),
            }
        )
        X1 = extract_features(df1, column_id=1, column_sort=2)
        self.assertEqual(len(X1), 2)

        df2 = pd.DataFrame(
            data={
                1: range(10),
                0: np.repeat([0, 1], 5),
                2: np.repeat([0, 1, 2, 3, 4], 2),
            }
        )
        X2 = extract_features(df2, column_id=0, column_sort=2)
        self.assertEqual(len(X2), 2)

        df3 = pd.DataFrame(
            data={
                0: range(10),
                2: np.repeat([0, 1], 5),
                1: np.repeat([0, 1, 2, 3, 4], 2),
            }
        )
        X3 = extract_features(df3, column_id=2, column_sort=1)
        self.assertEqual(len(X3), 2)

    def test_raises_mismatch_index_df_and_y_df_more(self):
        y = pd.Series(range(3), index=[1, 2, 3])
        df_dict = {
            "a": pd.DataFrame({"val": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}),
            "b": pd.DataFrame({"val": [5, 6, 7, 8, 12, 13], "id": [4, 4, 3, 3, 2, 2]}),
        }
        self.assertRaises(
            ValueError,
            extract_relevant_features,
            df_dict,
            y,
            None,
            None,
            None,
            "id",
            None,
            "val",
        )

    def test_raises_mismatch_index_df_and_y_y_more(self):
        y = pd.Series(range(4), index=[1, 2, 3, 4])
        df = pd.DataFrame({"val": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]})
        self.assertRaises(
            ValueError,
            extract_relevant_features,
            df,
            y,
            None,
            None,
            None,
            "id",
            None,
            "val",
        )

    def test_raises_y_not_series(self):
        y = np.arange(10)
        df_dict = {
            "a": pd.DataFrame({"val": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}),
            "b": pd.DataFrame({"val": [5, 6, 7, 8, 12, 13], "id": [4, 4, 3, 3, 2, 2]}),
        }
        self.assertRaises(
            AssertionError,
            extract_relevant_features,
            df_dict,
            y,
            None,
            None,
            None,
            "id",
            None,
            "val",
        )

    def test_raises_y_not_more_than_one_label(self):
        y = pd.Series(1, index=[1, 2, 3])
        df_dict = {
            "a": pd.DataFrame({"val": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}),
            "b": pd.DataFrame({"val": [5, 6, 7, 8, 12, 13], "id": [4, 4, 3, 3, 2, 2]}),
        }
        self.assertRaises(
            AssertionError,
            extract_relevant_features,
            df_dict,
            y,
            None,
            None,
            None,
            "id",
            None,
            "val",
        )



from unittest import TestCase

import pandas as pd
from dask import dataframe as dd

from tsfresh.convenience.bindings import dask_feature_extraction_on_chunk
from tsfresh.feature_extraction.settings import MinimalFCParameters


class DaskBindingsTestCase(TestCase):
    def test_feature_extraction(self):
        df = pd.DataFrame(
            {
                "my_id": [1, 1, 1, 2, 2, 2],
                "my_kind": ["a"] * 6,
                "my_value": [1, 2, 3, 4, 5, 6],
            }
        )

        df = dd.from_pandas(df, chunksize=3)

        df_grouped = df.groupby(["my_id", "my_kind"])

        features = dask_feature_extraction_on_chunk(
            df_grouped,
            column_id="my_id",
            column_kind="my_kind",
            column_value="my_value",
            column_sort=None,
            default_fc_parameters=MinimalFCParameters(),
        )

        features = features.compute()

        self.assertEqual(list(sorted(features.columns)), ["my_id", "value", "variable"])
        self.assertEqual(len(features), 2 * len(MinimalFCParameters()))



# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from unittest import TestCase

import dask.dataframe as dd
import pandas as pd

from tsfresh import extract_features, extract_relevant_features
from tsfresh.examples.driftbif_simulation import load_driftbif
from tsfresh.feature_extraction import MinimalFCParameters


class FeatureExtractionTestCase(TestCase):
    def setUp(self):
        df, y = load_driftbif(100, 10, classification=True, seed=42)

        df["my_id"] = df["id"].astype("str")
        del df["id"]

        self.df = df

    def test_pandas(self):
        df = self.df

        # Test shape and a single entry (to see if it works at all)
        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            column_value="value",
            default_fc_parameters=MinimalFCParameters(),
        )
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 20))

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            default_fc_parameters=MinimalFCParameters(),
        )
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 20))

        X = extract_features(
            df.drop(columns=["dimension"]),
            column_id="my_id",
            column_sort="time",
            default_fc_parameters=MinimalFCParameters(),
        )
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 10))

        X = extract_features(
            df.drop(columns=["dimension", "time"]),
            column_id="my_id",
            default_fc_parameters=MinimalFCParameters(),
        )
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 10))

    def test_pandas_no_pivot(self):
        df = self.df

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            column_value="value",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        )
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 20, 3))

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        )
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 20, 3))

        X = extract_features(
            df.drop(columns=["dimension"]),
            column_id="my_id",
            column_sort="time",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        )
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 10, 3))

        X = extract_features(
            df.drop(columns=["dimension", "time"]),
            column_id="my_id",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        )
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 10, 3))

    def test_dask(self):
        df = dd.from_pandas(self.df, npartitions=1)

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            column_value="value",
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 20))

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 20))

        X = extract_features(
            df.drop(columns=["dimension"]),
            column_id="my_id",
            column_sort="time",
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 10))

        X = extract_features(
            df.drop(columns=["dimension", "time"]),
            column_id="my_id",
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 10))

    def test_dask_no_pivot(self):
        df = dd.from_pandas(self.df, npartitions=1)

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            column_value="value",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 20, 3))

        X = extract_features(
            df,
            column_id="my_id",
            column_sort="time",
            column_kind="dimension",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 20, 3))

        X = extract_features(
            df.drop(columns=["dimension"]),
            column_id="my_id",
            column_sort="time",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 10, 3))

        X = extract_features(
            df.drop(columns=["dimension", "time"]),
            column_id="my_id",
            pivot=False,
            default_fc_parameters=MinimalFCParameters(),
        ).compute()
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(
            X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0],
            5.516e-05,
            4,
        )
        self.assertEqual(X.shape, (100 * 10, 3))



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

import os
import shutil
import tempfile
from unittest import TestCase

import numpy as np
from pandas import DataFrame, Series

from tsfresh import extract_features
from tsfresh.examples.robot_execution_failures import (
    download_robot_execution_failures,
    load_robot_execution_failures,
)


class RobotExecutionFailuresTestCase(TestCase):
    def setUp(self):
        self.temporary_folder = tempfile.mkdtemp()
        self.temporary_file = os.path.join(self.temporary_folder, "data")

        download_robot_execution_failures(file_name=self.temporary_file)
        self.X, self.y = load_robot_execution_failures(file_name=self.temporary_file)

    def tearDown(self):
        shutil.rmtree(self.temporary_folder)

    def test_characteristics_downloaded_robot_execution_failures(self):
        self.assertEqual(len(self.X), 1320)
        self.assertIsInstance(self.X, DataFrame)
        self.assertIsInstance(self.y, Series)
        self.assertCountEqual(
            ["id", "time", "F_x", "F_y", "F_z", "T_x", "T_y", "T_z"],
            list(self.X.columns),
        )

    def test_extraction_runs_through(self):
        df = extract_features(self.X[self.X.id < 3], column_id="id", column_sort="time")

        self.assertCountEqual(df.index.values, [1, 2])
        self.assertGreater(len(df), 0)

    def test_binary_target_is_default(self):
        _, y = load_robot_execution_failures(file_name=self.temporary_file)

        assert len(y.unique()) == 2

    def test_multilabel_target_on_request(self):
        _, y = load_robot_execution_failures(
            multiclass=True, file_name=self.temporary_file
        )

        assert len(y.unique()) > 2
        assert y.dtype == object



# todo: here should go a top level description (see for example the numpy top level __init__.py)

"""
At the top level we export the three most important submodules of tsfresh, which are:

    * :mod:`~tsfresh.extract_features`
    * :mod:`~tsfresh.select_features`
    * :mod:`~tsfresh.extract_relevant_features`
"""


import pkg_resources

try:
    __version__ = pkg_resources.get_distribution(__name__).version
except Exception:
    __version__ = "unknown"


# Set default logging handler to avoid "No handler found" warnings.
import logging
from logging import NullHandler

logging.getLogger(__name__).addHandler(NullHandler())


from tsfresh.convenience.relevant_extraction import (  # noqa: E402
    extract_relevant_features,
)
from tsfresh.feature_extraction import extract_features  # noqa: E402
from tsfresh.feature_selection import select_features  # noqa: E402
